@@ -1,6 +1,6 @@
 #include <libbonobo.h>
 #include <login-helper/login-helper.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
 
@@ -25,7 +25,7 @@ int
 main (int argc, char **argv)
 {
 	int          ret;
-	char        *obj_id, *display_name;
+	char        *obj_id;
 	BonoboObject *helper;
 	
 	if (!bonobo_init (&argc, argv))
@@ -125,7 +125,7 @@ test_get_raise_windows (LoginHelper *helper)
 
 
 void
-test_set_wm_dock ()
+test_set_wm_dock (void)
 {
   Atom atom_type[1], atom_window_type;
 
@@ -148,7 +148,7 @@ test_set_wm_dock ()
 }
 
 static void
-test_post_window ()
+test_post_window (void)
 {
         mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
